@@ -78,6 +78,7 @@ def test_simple_experiment(experiment_class):
                 )
             ]
         )
+        print(f"========================Model groups {num_mgs}")
         assert num_mgs > 0
 
         # 2. that model entries are present, and linked to model groups
@@ -93,6 +94,7 @@ def test_simple_experiment(experiment_class):
                 )
             ]
         )
+        print(f"========================Model {num_models}")
         assert num_models > 0
 
         # 3. predictions, linked to models for both training and testing predictions
@@ -109,10 +111,11 @@ def test_simple_experiment(experiment_class):
                     )
                 ]
             )
+            print(f"========================Predictions {num_predictions}")
             assert num_predictions > 0
 
         # 4. evaluations linked to predictions linked to models, for training and testing
-        
+
         for set_type in ("train", "test"):
             num_evaluations = len(
                 [
@@ -131,8 +134,9 @@ def test_simple_experiment(experiment_class):
                     )
                 ]
             )
+            print(f"========================Evaluations {num_evaluations}")
             assert num_evaluations > 0
-        
+
         # 5. subset evaluations linked to subsets and predictions linked to
         #    models, for training and testing
         for set_type in ("train", "test"):
