@@ -1,7 +1,7 @@
 import copy
 import os
-import numpy
-import pandas
+import numpy as np
+import pandas as pd
 
 from .plotting import plot_cats, plot_bounds
 
@@ -135,7 +135,7 @@ class SelectionRulePlotter:
     def regret_thresholds(self, regret_metric, regret_parameter):
         plot_min, plot_max = self.plot_bounds(regret_metric, regret_parameter)
         regret_threshold_dist = self.regret_threshold_dist(plot_min, plot_max)
-        return numpy.arange(plot_min, plot_max, regret_threshold_dist)
+        return np.arange(plot_min, plot_max, regret_threshold_dist)
 
     def create_plot_dataframe(
         self,
@@ -181,11 +181,11 @@ class SelectionRulePlotter:
                 accumulator.append(
                     {
                         "regret": regret_threshold,
-                        "pct_of_time": numpy.mean(distro),
+                        "pct_of_time": np.mean(distro),
                         "selection_rule": selection_rule.descriptive_name,
                     }
                 )
-        return pandas.DataFrame.from_records(accumulator)
+        return pd.DataFrame.from_records(accumulator)
 
     def plot_all_selection_rules(
         self,
