@@ -4,14 +4,16 @@ from .predictors import Predictor
 from .evaluation import ModelEvaluator
 from .individual_importance import IndividualImportanceCalculator, IndividualImportanceCalculatorNoOp
 from .model_grouping import ModelGrouper
-from .subsetters import Subsetter
+from .subsetters import Subsetter, SubsetterNoOp
 from .protected_groups_generators import ProtectedGroupsGenerator, ProtectedGroupsGeneratorNoOp
 from .utils import filename_friendly_hash
-import logging
+
+import verboselogs, logging
+logger = verboselogs.VerboseLogger(__name__)
+
 from collections import namedtuple
 
-import numpy
-import pandas
+import numpy as np
 
 TaskBatch = namedtuple('TaskBatch', ['parallelizable', 'tasks', 'description'])
 
@@ -36,12 +38,12 @@ class ModelTrainTester:
         self.predictor = predictor
         self.subsets = subsets
         self.replace = replace
-        self.protected_groups_generator = protected_groups_generator or ProtectedGroupsGeneratorNoOp()
+        self.protected_groups_generator = protected_groups_generator
         self.cohort_hash = cohort_hash
 
     def generate_task_batches(self, splits, grid_config, model_comment=None):
         train_test_tasks = []
-        logging.info("Generating train/test tasks for %s splits", len(splits))
+        logger.debug(f"Generating train/test tasks for {len(splits)} splits")
         for split in splits:
             train_store = self.matrix_storage_engine.get_store(split["train_uuid"])
             train_tasks = self.model_trainer.generate_train_tasks(
@@ -100,20 +102,26 @@ class ModelTrainTester:
             else:
                 # Last priority: Everything else. Maybe these are slow/non-parallelizable
                 batches[2].tasks.append(task)
-        logging.info("Split train/test tasks into three task batches. - each batch has models from all splits")
+        logger.verbose("Split train/test tasks into three task batches. - each batch has models from all splits")
         for batch_num, batch in enumerate(batches, 1):
-            logging.info("Batch %s: %s (%s tasks total)", batch_num, batch.description, len(batch.tasks))
+            logger.verbose(f"Batch {batch_num}: {batch.description} ({len(batch.tasks)} tasks total)")
+
+
+
         return batches
 
 
     def process_all_batches(self, task_batches):
-        # In the simple loop version here we ignore parallelizability and do everything serially
-        for batch in task_batches:
-            for task in batch.tasks:
+        for n_batch, batch in enumerate(task_batches, start=1):
+            logger.verbose(f"Processing '{batch.description}' [{n_batch} of {len(task_batches)} batches]")
+            for n_task, task in enumerate(batch.tasks, start=1):
+                logger.verbose(f"Processing task [{n_task} of {len(batch.tasks)}] from {batch.description}")
                 self.process_task(**task)
+                logger.verbose(f"Task {n_task} from {batch.description} completed")
+            logger.success(f"Batch '{batch.description}' completed")
 
     def process_task(self, test_store, train_store, train_kwargs):
-        logging.info("Beginning train task %s", train_kwargs)
+        logger.verbose(f"Training {train_kwargs.get('class_path')}({train_kwargs.get('parameters')}) [{train_kwargs.get('model_hash')}] on train matrix {train_store.uuid}")
 
         # If the matrices and train labels are OK, train and test the model!
         with self.model_trainer.cache_models(), test_store.cache(), train_store.cache():
@@ -123,62 +131,56 @@ class ModelTrainTester:
             # If the train or test design matrix empty, or if the train store only
             # has one label value, skip training the model.
             if train_store.empty:
-                logging.warning(
-                    """Train matrix for split %s was empty,
-                no point in training this model. Skipping
-                """,
-                    train_store.uuid
+                logger.notice(
+                    f"""Train matrix for split {train_store.uuid} was empty,
+                    no point in training this model. Skipping
+                    """
                 )
                 return
+
             if len(train_store.labels.unique()) == 1:
-                logging.warning(
-                    """Train Matrix for split %s had only one
-                unique value, no point in training this model. Skipping
-                """,
-                    train_store.uuid,
+                logger.notice(
+                    f"""Train Matrix for split {train_store.uuid} had only one
+                    unique value, no point in training this model. Skipping
+                    """
                 )
                 return
+
             if test_store.empty:
-                logging.warning(
-                    """Test matrix for uuid %s
-                was empty, no point in generating predictions. Not processing train/test task.
-                """,
-                    test_store.uuid,
+                logger.notice(
+                    f"""Test matrix for uuid {test_store.uuid}
+                    was empty, no point in generating predictions. Not processing train/test task.
+                    """
                 )
                 return
 
             model_id = self.model_trainer.process_train_task(**train_kwargs)
-            if not model_id:
-                logging.warning("No model id returned from ModelTrainer.process_train_task, "
-                                "training unsuccessful. Not attempting to test")
-                return
-            logging.info("Trained task %s and got model id %s", train_kwargs, model_id)
-            as_of_dates = test_store.as_of_dates
-            logging.info(
-                "Testing and scoring model id %s with test matrix %s. "
-                "as_of_times min: %s max: %s num: %s",
-                model_id,
-                test_store.uuid,
-                min(as_of_dates),
-                max(as_of_dates),
-                len(as_of_dates),
-            )
 
+            if not model_id:
+                logger.warning("Training unsuccessful for {train_kwargs.get('class_path')}({train_kwargs.get('parameters')}) [{train_kwargs.get('model_hash')}] on train matrix {train_store.uuid}. "
+                               "No model id returned.  Not attempting to test it")
+                return
+
+            logger.success(f"Trained model id {model_id}: {train_kwargs.get('class_path')}({train_kwargs.get('parameters')}) [{train_kwargs.get('model_hash')}] on train matrix {train_store.uuid}. ")
+
+            # Storing individual importances (if any)
             self.individual_importance_calculator.calculate_and_save_all_methods_and_dates(
                 model_id, test_store
             )
 
+            as_of_dates = test_store.as_of_dates
+            logger.debug(
+                f"Testing and evaluating model {model_id}  {train_kwargs.get('class_path')}({train_kwargs.get('parameters')}) [{train_kwargs.get('model_hash')}] "
+                f"on test matrix {test_store.uuid}. ")
+            logger.spam(f"as_of_times min: {min(as_of_dates)} max: {max(as_of_dates)} num: {len(as_of_dates)}")
+
+
             # Generate predictions for the testing data then training data
             for store in (test_store, train_store):
-                predictions_proba = numpy.array(None)
-                protected_df = None
-                if self.replace:
-                    logging.info(
-                        "Replace flag set; generating new predictions and evaluations for"
-                        "matrix %s-%s, and model %s",
-                        store.uuid,
-                        store.matrix_type,
-                        model_id
+                if self.replace or self.model_evaluator.needs_evaluations(store, model_id):
+                    logger.spam(
+                        f"Generating new predictions for "
+                        f"{store.matrix_type.string_name} matrix {store.uuid}, and model {model_id} to make evaluation",
                     )
 
                     predictions_proba = self.predictor.predict(
@@ -188,38 +190,46 @@ class ModelTrainTester:
                         train_matrix_columns=train_store.columns(),
                     )
 
+                    logger.debug(f"Predictions generated for {store.matrix_type.string_name} matrix {store.uuid} using model {model_id}")
+
+
+                    protected_df = self.protected_groups_generator.as_dataframe(
+                        as_of_dates=store.as_of_dates,
+                        cohort_hash=self.cohort_hash,
+                    )
+
+                    logger.spam(
+                        f"Evaluating model {model_id} on {store.matrix_type.string_name} matrix {store.uuid} "
+                    )
+
+                    self.model_evaluator.evaluate(
+                        predictions_proba=predictions_proba,
+                        matrix_store=store,
+                        model_id=model_id,
+                        subset=None,
+                        protected_df=protected_df
+                    )
+
+                    logger.info(
+                        f"Model {model_id} evaluation on {store.matrix_type.string_name} matrix {store.uuid} completed."
+                    )
+
+                else:
+                    logger.notice(
+                        f"The evaluations needed for {store.matrix_type.string_name} matrix {store.uuid} and "
+                        f"model {model_id} are all present"
+                        f"in db from a previous run (or none needed at all), so skipping!",
+                    )
+
+
                 for subset in self.subsets:
-                    if self.replace or self.model_evaluator.needs_evaluations(
-                        store, model_id, filename_friendly_hash(subset)
-                    ):
-                        logging.info(
-                            "Evaluating matrix %s-%s, subset %s, and model %s",
-                            store.uuid,
-                            store.matrix_type,
-                            filename_friendly_hash(subset),
-                            model_id,
+                    subset_hash = filename_friendly_hash(subset)
+                    if self.replace or self.model_evaluator.needs_evaluations(store, model_id, subset_hash):
+
+                        logger.spam(
+                            f"Evaluating {store.matrix_type.string_name} matrix {store.uuid}, subset {subset_hash}, and model {model_id}"
                         )
 
-                        if not predictions_proba.any():
-                            logging.info(
-                                "Generating new predictions for"
-                                "matrix %s-%s, and model %s to make evaluation",
-                                store.uuid,
-                                store.matrix_type,
-                                model_id
-                            )
-
-                            predictions_proba = self.predictor.predict(
-                                model_id,
-                                store,
-                                misc_db_parameters=dict(),
-                                train_matrix_columns=train_store.columns(),
-                            )
-                        if protected_df is None:
-                            protected_df = self.protected_groups_generator.as_dataframe(
-                                as_of_dates=store.as_of_dates,
-                                cohort_hash=self.cohort_hash,
-                            )
 
                         self.model_evaluator.evaluate(
                             predictions_proba=predictions_proba,
@@ -229,17 +239,16 @@ class ModelTrainTester:
                             protected_df=protected_df
                         )
 
+                        logger.info(
+                            f"Model {model_id} evaluation on subset {filename_friendly_hash(subset)} of {store.matrix_type.string_name} matrix {store.uuid} completed."
+                        )
+
                     else:
-                        logging.info(
-                            "The evaluations needed for matrix %s-%s, subset %s, and "
-                            "model %s are all present"
-                            "in db from a previous run (or none needed at all), so skipping!",
-                            store.uuid,
-                            store.matrix_type,
-                            filename_friendly_hash(subset),
-                            model_id
+                        logger.notice(
+                            f"The evaluations needed for {store.matrix_type.string_name} matrix {store.uuid}, subset {filename_friendly_hash(subset)}, and "
+                            f"model {model_id} are all present"
+                            f"in db from a previous run (or none needed at all), so skipping!",
                         )
-                self.predictor.update_db_with_ranks(model_id, store.uuid, store.matrix_type)
 
 
 __all__ = (
