@@ -1,5 +1,6 @@
-# -*- coding: utf-8 -*-
-import logging
+import verboselogs, logging
+logger = verboselogs.VerboseLogger(__name__)
+
 from numbers import Number
 from itertools import product, chain
 import sqlalchemy.sql.expression as ex
@@ -495,7 +496,7 @@ class Aggregation:
                         raise ValueError("Duplicate feature column name found: ", col.name)
                     lookup[col.name] = agg
         return lookup
-            
+
     def _col_prefix(self, group):
         """
         Helper for creating a column prefix for the group
@@ -719,7 +720,7 @@ class Aggregation:
             if col in impute_cols:
 
                 # we don't want to add redundant imputation flags. for a given source
-                # column and time interval, all of the functions will have identical 
+                # column and time interval, all of the functions will have identical
                 # sets of rows that needed imputation
                 # to reliably merge these, we lookup the original aggregate that produced
                 # the function, and see its available functions. we expect exactly one of
@@ -733,7 +734,7 @@ class Aggregation:
                     else:
                         impflag_basecol = col.rstrip('_' + used_function)
                 else:
-                    logging.warning("Imputation flag merging is not implemented for "
+                    logger.warning("Imputation flag merging is not implemented for "
                                     "AggregateExpression objects that don't define an aggregate "
                                     "function (e.g. composites)")
                     impflag_basecol = col
