@@ -36,13 +36,22 @@ schemas = (
 event.listen(Base.metadata, "before_create", DDL(schemas))
 
 group_proc_filename = os.path.join(
-    os.path.dirname(__file__), "model_group_stored_procedure.sql"
+    os.path.dirname(__file__), "sql", "model_group_stored_procedure.sql"
 )
 with open(group_proc_filename) as fd:
     stmt = fd.read()
 
 event.listen(Base.metadata, "before_create", DDL(stmt))
 
+nuke_triage_filename = os.path.join(
+    os.path.dirname(__file__), "sql", "nuke_triage.sql"
+)
+with open(nuke_triage_filename) as fd:
+    stmt = fd.read()
+
+
+event.listen(Base.metadata, "before_create", DDL(stmt.replace('%', '%%')))
+
 
 class Experiment(Base):
 
@@ -59,6 +68,51 @@ class Experiment(Base):
     matrices_needed = Column(Integer)
     grid_size = Column(Integer)
     models_needed = Column(Integer)
+    random_seed = Column(Integer)
+
+
+class ExperimentRunStatus(enum.Enum):
+    started = 1
+    completed = 2
+    failed = 3
+
+
+class ExperimentRun(Base):
+
+    __tablename__ = "experiment_runs"
+    __table_args__ = {"schema": "triage_metadata"}
+
+    run_id = Column("id", Integer, primary_key=True)
+    start_time = Column(DateTime)
+    start_method = Column(String)
+    git_hash = Column(String)
+    triage_version = Column(String)
+    python_version = Column(String)
+    experiment_hash = Column(
+        String,
+        ForeignKey("triage_metadata.experiments.experiment_hash")
+    )
+    platform = Column(Text)
+    os_user = Column(Text)
+    working_directory = Column(Text)
+    ec2_instance_type = Column(Text)
+    log_location = Column(Text)
+    experiment_class_path = Column(Text)
+    experiment_kwargs = Column(JSONB)
+    installed_libraries = Column(ARRAY(Text))
+    matrix_building_started = Column(DateTime)
+    matrices_made = Column(Integer, default=0)
+    matrices_skipped = Column(Integer, default=0)
+    matrices_errored = Column(Integer, default=0)
+    model_building_started = Column(DateTime)
+    models_made = Column(Integer, default=0)
+    models_skipped = Column(Integer, default=0)
+    models_errored = Column(Integer, default=0)
+    last_updated_time = Column(DateTime, onupdate=datetime.datetime.now)
+    current_status = Column(Enum(ExperimentRunStatus))
+    stacktrace = Column(Text)
+    random_seed = Column(Integer)
+    experiment_rel = relationship("Experiment")
 
 
 class Subset(Base):
@@ -155,6 +209,9 @@ class Model(Base):
     built_by_experiment = Column(
         String, ForeignKey("triage_metadata.experiments.experiment_hash")
     )
+    built_in_experiment_run = Column(
+        Integer, ForeignKey("triage_metadata.experiment_runs.id")
+    )
     train_end_time = Column(DateTime)
     test = Column(Boolean)
     train_matrix_uuid = Column(Text, ForeignKey("triage_metadata.matrices.matrix_uuid"))
@@ -496,46 +553,10 @@ class TrainAequitas(Base):
     model_rel = relationship("Model")
 
 
+hash_partitioning_filename = os.path.join(
+    os.path.dirname(__file__), "sql", "predictions_hash_partitioning.sql"
+)
+with open(hash_partitioning_filename) as fd:
+    stmt = fd.read()
 
-class ExperimentRunStatus(enum.Enum):
-    started = 1
-    completed = 2
-    failed = 3
-
-
-class ExperimentRun(Base):
-
-    __tablename__ = "experiment_runs"
-    __table_args__ = {"schema": "triage_metadata"}
-
-    run_id = Column("id", Integer, primary_key=True)
-    start_time = Column(DateTime)
-    start_method = Column(String)
-    git_hash = Column(String)
-    triage_version = Column(String)
-    python_version = Column(String)
-    experiment_hash = Column(
-        String,
-        ForeignKey("triage_metadata.experiments.experiment_hash")
-    )
-    platform = Column(Text)
-    os_user = Column(Text)
-    working_directory = Column(Text)
-    ec2_instance_type = Column(Text)
-    log_location = Column(Text)
-    experiment_class_path = Column(Text)
-    experiment_kwargs = Column(JSONB)
-    installed_libraries = Column(ARRAY(Text))
-    matrix_building_started = Column(DateTime)
-    matrices_made = Column(Integer, default=0)
-    matrices_skipped = Column(Integer, default=0)
-    matrices_errored = Column(Integer, default=0)
-    model_building_started = Column(DateTime)
-    models_made = Column(Integer, default=0)
-    models_skipped = Column(Integer, default=0)
-    models_errored = Column(Integer, default=0)
-    last_updated_time = Column(DateTime, onupdate=datetime.datetime.now)
-    current_status = Column(Enum(ExperimentRunStatus))
-    stacktrace = Column(Text)
-
-    experiment_rel = relationship("Experiment")
+event.listen(Base.metadata, "after_create", DDL(stmt))
