@@ -1,4 +1,6 @@
-import logging
+import verboselogs, logging
+logger = verboselogs.VerboseLogger(__name__)
+
 from abc import ABC, abstractmethod
 import cProfile
 import marshal
@@ -25,7 +27,7 @@ from triage.component.architect.planner import Planner
 from triage.component.architect.builders import MatrixBuilder
 from triage.component.architect.entity_date_table_generators import (
     EntityDateTableGenerator,
-    EntityDateTableGeneratorNoOp,
+    CohortTableGeneratorNoOp,
 )
 from triage.component.timechop import Timechop
 from triage.component import results_schema
@@ -37,7 +39,8 @@ from triage.component.catwalk import (
     IndividualImportanceCalculatorNoOp,
     ModelGrouper,
     ModelTrainTester,
-    Subsetter
+    Subsetter,
+    SubsetterNoOp
 )
 from triage.component.catwalk.protected_groups_generators import (
     ProtectedGroupsGenerator,
@@ -136,9 +139,6 @@ class ExperimentBase(ABC):
         self._check_config_version(config)
         self.config = config
 
-        self.config['random_seed'] = self.config.get('random_seed', random.randint(1,1e7))
-
-        random.seed(self.config['random_seed'])
 
         self.project_storage = ProjectStorage(project_path)
         self.model_storage_engine = ModelStorageEngine(self.project_storage)
@@ -146,15 +146,44 @@ class ExperimentBase(ABC):
             self.project_storage, matrix_storage_class
         )
         self.project_path = project_path
+        logger.verbose(f"Matrices and trained models will be saved in {self.project_path}")
         self.replace = replace
+        if self.replace:
+            logger.notice(f"Replace flag is set to true. Matrices, models, "
+                          "evaluations and predictions (if exist) will be replaced")
+
         self.save_predictions = save_predictions
+        if not self.save_predictions:
+            logger.notice(f"Save predictions flag is set to false. "
+                          "Predictions won't be stored in the predictions "
+                          "table. This will decrease both the running time "
+                          "of an experiment and also decrease the space needed in the db")
+
         self.skip_validation = skip_validation
+        if self.skip_validation:
+           logger.notice(f"Skip validation flag is set to true. "
+                         "The experiment config file specified won't be validated. "
+                         "This will reduce (a little) the running time of the experiment, "
+                         "but has some potential risks, e.g. the experiment could fail"
+                         "after some time due some misconfiguration. Proceed with care.")
+
         self.db_engine = db_engine
         results_schema.upgrade_if_clean(dburl=self.db_engine.url)
 
         self.features_schema_name = "features"
+
         self.materialize_subquery_fromobjs = materialize_subquery_fromobjs
+        if not self.materialize_subquery_fromobjs:
+            logger.notice("Materialize from_objs is set to false. "
+                          "The from_objs will be calculated on the fly every time.")
+
         self.features_ignore_cohort = features_ignore_cohort
+        if self.features_ignore_cohort:
+            logger.notice("Features will be calculated for all the entities "
+                          "(i.e. ignoring cohort) this setting will have the effect "
+                          "that more db space will be used, but potentially could save "
+                          "time is you are running different similar experiments with "
+                          "different cohorts.")
 
         # only fill default values for full runs
         if not partial_run:
@@ -170,42 +199,56 @@ class ExperimentBase(ABC):
         if grid_config is not None:
             self.config['grid_config'] = grid_config
 
+
+        if not self.config.get('random_seed', None):
+            logger.notice("Random seed not specified. A random seed will be provided. "
+                          "This could have interesting side effects, "
+                          "e.g. new models per model group are trained, "
+                          "tested and evaluated everytime that you run this experiment configuration")
+
+
+        self.random_seed = self.config.pop('random_seed', random.randint(1,1e7))
+
+        logger.verbose(f"Using random seed [{self.random_seed}] for running the experiment")
+        random.seed(self.random_seed)
+
         ###################### RUBICON ######################
 
-        self.experiment_hash = save_experiment_and_get_hash(self.config, self.db_engine)
+        self.experiment_hash = save_experiment_and_get_hash(self.config, self.random_seed, self.db_engine)
+        logger.debug(f"Experiment hash [{self.experiment_hash}] assigned")
         self.run_id = initialize_tracking_and_get_run_id(
             self.experiment_hash,
             experiment_class_path=classpath(self.__class__),
+            random_seed=self.random_seed,
             experiment_kwargs=experiment_kwargs,
             db_engine=self.db_engine
         )
+        logger.debug(f"Experiment run id [{self.run_id}] assigned")
+
         self.initialize_components()
 
         self.cleanup = cleanup
         if self.cleanup:
-            logging.info(
-                "cleanup is set to True, so intermediate tables (labels and cohort) "
+            logger.notice(
+                "Cleanup is set to true, so intermediate tables (labels and cohort) "
                 "will be removed after matrix creation and subset tables will be "
                 "removed after model training and testing"
             )
-        else:
-            logging.info(
-                "cleanup is set to False, so intermediate tables (labels, cohort, and subsets) "
-                "will not be removed"
-            )
+
         self.cleanup_timeout = (
             self.cleanup_timeout if cleanup_timeout is None else cleanup_timeout
         )
+
         self.profile = profile
-        logging.info("Generate profiling stats? (profile option): %s", self.profile)
+        if self.profile:
+            logger.spam("Profiling will be stored using cProfile")
 
     def _check_config_version(self, config):
         if "config_version" in config:
             config_version = config["config_version"]
         else:
-            logging.warning(
+            raise ValueError(
                 "config_version key not found in experiment config. "
-                "Assuming v1, which may not be correct"
             )
             config_version = "v1"
         if config_version != CONFIG_VERSION:
@@ -240,16 +283,15 @@ class ExperimentBase(ABC):
                 replace=self.replace
             )
         else:
-            logging.warning(
+            logger.warning(
                 "cohort_config missing or unrecognized. Without a cohort, "
                 "you will not be able to make matrices, perform feature imputation, "
                 "or save time by only computing features for that cohort."
             )
             self.features_ignore_cohort = True
             self.cohort_table_name = "cohort_{}".format(self.experiment_hash)
-            self.cohort_table_generator = EntityDateTableGeneratorNoOp()
+            self.cohort_table_generator = CohortTableGeneratorNoOp()
 
-        self.subsets = [None] + self.config.get("scoring", {}).get("subsets", [])
 
         if "label_config" in self.config:
             label_config = self.config["label_config"]
@@ -266,7 +308,7 @@ class ExperimentBase(ABC):
         else:
             self.labels_table_name = "labels_{}".format(self.experiment_hash)
             self.label_generator = LabelGeneratorNoOp()
-            logging.warning(
+            logger.warning(
                 "label_config missing or unrecognized. Without labels, "
                 "you will not be able to make matrices."
             )
@@ -286,9 +328,9 @@ class ExperimentBase(ABC):
             )
         else:
             self.protected_groups_generator = ProtectedGroupsGeneratorNoOp()
-            logging.warning(
-                "bias_audit_config missing or unrecognized. Without protected groups, "
-                "you will not audit your models for bias and fairness."
+            logger.notice(
+                "bias_audit_config missing in the configuration file or unrecognized. "
+                "Without protected groups, you will not audit your models for bias and fairness."
             )
 
         self.feature_dictionary_creator = FeatureDictionaryCreator(
@@ -339,11 +381,16 @@ class ExperimentBase(ABC):
             run_id=self.run_id,
         )
 
-        self.subsetter = Subsetter(
-            db_engine=self.db_engine,
-            replace=self.replace,
-            as_of_times=self.all_as_of_times
-        )
+        self.subsets = self.config.get("scoring", {}).get("subsets", [])
+        if self.subsets:
+            self.subsetter = Subsetter(
+                db_engine=self.db_engine,
+                replace=self.replace,
+                as_of_times=self.all_as_of_times
+            )
+        else:
+            self.subsetter = SubsetterNoOp()
+            logger.notice("scoring.subsets missing in the configuration file or unrecognized. No subsets will be generated")
 
         self.trainer = ModelTrainer(
             experiment_hash=self.experiment_hash,
@@ -372,8 +419,8 @@ class ExperimentBase(ABC):
             )
         else:
             self.individual_importance_calculator = IndividualImportanceCalculatorNoOp()
-            logging.warning(
-                "individual_importance  missing or unrecognized."
+            logger.notice(
+                "individual_importance missing in the configuration file or unrecognized, "
                 "you will not be able to do analysis on individual feature importances."
             )
 
@@ -432,9 +479,10 @@ class ExperimentBase(ABC):
 
         """
         split_definitions = self.chopper.chop_time()
-        logging.info("Computed and stored split definitions: %s", split_definitions)
-        logging.info("\n----TIME SPLIT SUMMARY----\n")
-        logging.info("Number of time splits: {}".format(len(split_definitions)))
+        logger.verbose(f"Computed and stored temporal split definitions")
+        logger.debug(f"Temporal split definitions: {split_definitions}")
+        logger.spam("\n----TIME SPLIT SUMMARY----\n")
+        logger.spam("Number of time splits: {len(split_definitions)}")
         for split_index, split in enumerate(split_definitions):
             train_times = split["train_matrix"]["as_of_times"]
             test_times = [
@@ -442,18 +490,10 @@ class ExperimentBase(ABC):
                 for test_matrix in split["test_matrices"]
                 for as_of_time in test_matrix["as_of_times"]
             ]
-            logging.info(
-                """Split index {}:
-            Training as_of_time_range: {} to {} ({} total)
-            Testing as_of_time range: {} to {} ({} total)\n\n""".format(
-                    split_index,
-                    min(train_times),
-                    max(train_times),
-                    len(train_times),
-                    min(test_times),
-                    max(test_times),
-                    len(test_times),
-                )
+            logger.spam(
+                f"""Split index {split_index}:"""
+                f"""Training as_of_time_range: {min(train_times)} to {max(train_times)} ({len(train_times)} total)"""
+                f"""Testing as_of_time range: {min(test_times)} to {max(test_times)} ({len(test_times)} total)\n\n"""
             )
 
         with self.get_for_update() as experiment:
@@ -469,30 +509,27 @@ class ExperimentBase(ABC):
         Returns: (list) of datetimes
 
         """
+        logger.spam("Calculating all the as_of_times")
         all_as_of_times = []
         for split in self.split_definitions:
             all_as_of_times.extend(split["train_matrix"]["as_of_times"])
-            logging.debug(
-                "Adding as_of_times from train matrix: %s",
-                split["train_matrix"]["as_of_times"],
+            logger.spam(
+                f'Adding as_of_times from train matrix: {split["train_matrix"]["as_of_times"]}'
             )
             for test_matrix in split["test_matrices"]:
-                logging.debug(
-                    "Adding as_of_times from test matrix: %s",
-                    test_matrix["as_of_times"],
+                logger.spam(
+                    f'Adding as_of_times from test matrix: {test_matrix["as_of_times"]}',
                 )
                 all_as_of_times.extend(test_matrix["as_of_times"])
 
-        logging.info(
-            "Computed %s total as_of_times for label and feature generation",
-            len(all_as_of_times),
+        logger.spam(
+            f"Computed {len(all_as_of_times)} total as_of_times for label and feature generation",
         )
         distinct_as_of_times = list(set(all_as_of_times))
-        logging.info(
-            "Computed %s distinct as_of_times for label and feature generation",
-            len(distinct_as_of_times),
+        logger.debug(
+            f"Computed {len(distinct_as_of_times)} distinct as_of_times for label and feature generation",
         )
-        logging.info(
+        logger.spam(
             "You can view all as_of_times by inspecting `.all_as_of_times` on this Experiment"
         )
         with self.get_for_update() as experiment:
@@ -506,9 +543,9 @@ class ExperimentBase(ABC):
         Returns: (list) of ``collate.Aggregation`` objects
 
         """
-        logging.info("Creating collate aggregations")
+        logger.info("Creating collate aggregations")
         if "feature_aggregations" not in self.config:
-            logging.warning("No feature_aggregation config is available")
+            logger.warning("No feature_aggregation config is available")
             return []
         aggregations = self.feature_generator.aggregations(
             feature_aggregation_config=self.config["feature_aggregations"],
@@ -530,8 +567,8 @@ class ExperimentBase(ABC):
             being lists of SQL commands
 
         """
-        logging.info(
-            "Calculating feature tasks for %s as_of_times", len(self.all_as_of_times)
+        logger.spam(
+            f"Calculating feature aggregation tasks for {len(self.all_as_of_times)} as_of_times"
         )
         return self.feature_generator.generate_all_table_tasks(
             self.collate_aggregations, task_type="aggregation"
@@ -548,8 +585,8 @@ class ExperimentBase(ABC):
             being lists of SQL commands
 
         """
-        logging.info(
-            "Calculating feature tasks for %s as_of_times", len(self.all_as_of_times)
+        logger.spam(
+            f"Calculating feature imputation tasks for {len(self.all_as_of_times)} as_of_times"
         )
         return self.feature_generator.generate_all_table_tasks(
             self.collate_aggregations, task_type="imputation"
@@ -570,7 +607,7 @@ class ExperimentBase(ABC):
                 self.collate_aggregations
             ),
         )
-        logging.info("Computed master feature dictionary: %s", result)
+        logger.debug(f"Computed master feature dictionary: {result}")
         with self.get_for_update() as experiment:
             experiment.total_features = sum(1 for _feature in itertools.chain.from_iterable(result.values()))
         return result
@@ -604,10 +641,10 @@ class ExperimentBase(ABC):
 
         """
         if not table_has_data(self.cohort_table_name, self.db_engine):
-            logging.warning("cohort table is not populated, cannot build any matrices")
+            logger.warning("cohort table is not populated, cannot build any matrices")
             return {}
         if not table_has_data(self.labels_table_name, self.db_engine):
-            logging.warning("labels table is not populated, cannot build any matrices")
+            logger.warning("labels table is not populated, cannot build any matrices")
             return {}
         (updated_split_definitions, matrix_build_tasks) = self.planner.generate_plans(
             self.split_definitions, self.feature_dicts
@@ -652,15 +689,20 @@ class ExperimentBase(ABC):
 
         Results are stored in the database, not returned
         """
+        logger.info("Setting up labels")
         self.label_generator.generate_all_labels(
             self.labels_table_name, self.all_as_of_times, self.all_label_timespans
         )
+        logger.success(f"Labels setted up in the table {self.labels_table_name} successfully ")
 
     @experiment_entrypoint
     def generate_cohort(self):
+        logger.info("Setting up cohort")
         self.cohort_table_generator.generate_entity_date_table(
             as_of_dates=self.all_as_of_times
         )
+        logger.success(f"Cohort setted up in the table {self.cohort_table_name} successfully")
+
 
     @experiment_entrypoint
     def generate_protected_groups(self):
@@ -672,13 +714,9 @@ class ExperimentBase(ABC):
             self.all_as_of_times, self.cohort_table_name, self.cohort_hash
         )
 
-    def generate_subset(self, subset_hash):
-        self.subsets["subset_hash"].subset_table_generator.generate_entity_date_table(
-            as_of_dates=self.all_as_of_times
-        )
 
     def log_split(self, split_num, split):
-        logging.info(
+        logger.info(
             "Starting train/test for %s out of %s: train range: %s to %s",
             split_num + 1,
             len(self.full_matrix_definitions),
@@ -704,20 +742,22 @@ class ExperimentBase(ABC):
 
     @experiment_entrypoint
     def generate_preimputation_features(self):
+        logger.info("Creating features tables (before imputation) ")
         self.process_query_tasks(self.feature_aggregation_table_tasks)
-        logging.info(
-            "Finished running preimputation feature queries. The final results are in tables: %s",
-            ",".join(agg.get_table_name() for agg in self.collate_aggregations),
+        logger.success(
+            f"Features (before imputation) were stored in the tables "
+            f"{','.join(agg.get_table_name() for agg in self.collate_aggregations)} "
+            f"successfully"
         )
 
     @experiment_entrypoint
     def impute_missing_features(self):
+        logger.info("Imputing missing values in features")
         self.process_query_tasks(self.feature_imputation_table_tasks)
-        logging.info(
-            "Finished running postimputation feature queries. The final results are in tables: %s",
-            ",".join(
-                agg.get_table_name(imputed=True) for agg in self.collate_aggregations
-            ),
+        logger.success(
+            f"Imputed features were stored in the tables "
+            f"{','.join(agg.get_table_name(imputed=True) for agg in self.collate_aggregations)} "
+            f"successfully"
         )
 
     def build_matrices(self):
@@ -726,35 +766,32 @@ class ExperimentBase(ABC):
             self.matrix_build_tasks.keys(),
             self.db_engine
         )
+        logger.info("Building matrices")
+        logger.verbose(f"It is necessary to build {len(self.matrix_build_tasks.keys())} matrices")
         with self.get_for_update() as experiment:
             experiment.matrices_needed = len(self.matrix_build_tasks.keys())
         record_matrix_building_started(self.run_id, self.db_engine)
         self.process_matrix_build_tasks(self.matrix_build_tasks)
+        logger.success(f"Matrices were stored in {self.project_path}/matrices successfully")
+
 
     @experiment_entrypoint
     def generate_matrices(self):
-        logging.info("Creating cohort")
+        self.all_as_of_times # Forcing the calculation of all the as of times, so the logging makes more sense
         self.generate_cohort()
-        logging.info("Creating labels")
         self.generate_labels()
-        logging.info("Creating feature aggregation tables")
         self.generate_preimputation_features()
-        logging.info("Creating feature imputation tables")
         self.impute_missing_features()
-        logging.info("Building all matrices")
         self.build_matrices()
 
     @experiment_entrypoint
     def generate_subsets(self):
-        if self.subsets:
-            logging.info("Beginning subset generation")
-            self.process_subset_tasks(self.subset_tasks)
-        else:
-            logging.info("No subsets found. Proceeding to training and testing models")
+        self.process_subset_tasks(self.subset_tasks)
 
     def _all_train_test_batches(self):
+        """ A batch is a model_group to be train, test and evaluated """
         if "grid_config" not in self.config:
-            logging.warning(
+            logger.warning(
                 "No grid_config was passed in the experiment config. No models will be trained"
             )
             return
@@ -767,19 +804,18 @@ class ExperimentBase(ABC):
 
     @experiment_entrypoint
     def train_and_test_models(self):
-        self.generate_subsets()
-        logging.info("Creating protected groups table")
-        self.generate_protected_groups()
         batches = self._all_train_test_batches()
         if not batches:
-            logging.warning("No train/test tasks found, so no training to do")
+            logger.notice("No train/test tasks found, so no training to do")
             return
 
         with self.get_for_update() as experiment:
             experiment.grid_size = sum(
                 1 for _param in self.trainer.flattened_grid_config(self.config.get('grid_config')))
+            logger.info(f"{experiment.grid_size} models groups will be trained, tested and evaluated")
 
-        logging.info("%s train/test batches found. Beginning training.", len(batches))
+        logger.info(f"Training, testing and evaluating models")
+        logger.verbose(f"{len(batches)} train/test tasks found.")
         model_hashes = set(task['train_kwargs']['model_hash'] for batch in batches for task in batch.tasks)
         associate_models_with_experiment(
             self.experiment_hash,
@@ -790,6 +826,7 @@ class ExperimentBase(ABC):
             experiment.models_needed = len(model_hashes)
         record_model_building_started(self.run_id, self.db_engine)
         self.process_train_test_batches(batches)
+        logger.success("Training, testing and evaluatiog models completed")
 
     def validate(self, strict=True):
         ExperimentValidator(self.db_engine, strict=strict).run(self.config)
@@ -798,53 +835,56 @@ class ExperimentBase(ABC):
         if not self.skip_validation:
             self.validate()
 
-        logging.info("Generating matrices")
         try:
             self.generate_matrices()
+            self.generate_subsets()
+            self.generate_protected_groups()
             self.train_and_test_models()
+            self._log_end_of_run_report()
+        except Exception:
+            logger.error("Uh oh... Houston we have a problem")
+            raise
         finally:
             if self.cleanup:
                 self.clean_up_matrix_building_tables()
                 self.clean_up_subset_tables()
-            logging.info("Experiment complete")
-            self._log_end_of_run_report()
+                logger.notice("Cleanup flag was set to True, so label, cohort and subset tables were deleted")
 
     def _log_end_of_run_report(self):
-        missing_models = missing_model_hashes(self.experiment_hash, self.db_engine)
-        if len(missing_models) > 0:
-            logging.info("Found %s missing model hashes."
-                         "This means that they were supposed to either be trained or reused"
-                         "by this experiment but are not present in the models table."
-                         "Inspect the logs for any training errors. Full list: %s",
-                         len(missing_models),
-                         missing_models
-                         )
-        else:
-            logging.info("All models that were supposed to be trained were trained. Awesome!")
-
         missing_matrices = missing_matrix_uuids(self.experiment_hash, self.db_engine)
         if len(missing_matrices) > 0:
-            logging.info("Found %s missing matrix uuids."
-                         "This means that they were supposed to either be build or reused"
-                         "by this experiment but are not present in the matrices table."
-                         "Inspect the logs for any matrix building errors. Full list: %s",
-                         len(missing_matrices),
-                         missing_matrices
-                         )
+            logger.notice(f"Found {len(missing_matrices)} missing matrix uuids."
+                          f"This means that they were supposed to either be build or reused"
+                          f"by this experiment but are not present in the matrices table."
+                          f"Inspect the logs for any matrix building errors. Full list: {missing_matrices}",
+                          )
         else:
-            logging.info("All matrices that were supposed to be build were built. Awesome!")
+            logger.success("All matrices that were supposed to be build were built. Awesome!")
+
+        missing_models = missing_model_hashes(self.experiment_hash, self.db_engine)
+        if len(missing_models) > 0:
+            logger.notice(f"Found {len(missing_models)} missing model hashes. "
+                          f"This means that they were supposed to either be trained or reused "
+                          f"by this experiment but are not present in the models table. "
+                          f"Inspect the logs for any training errors. Full list: {missing_models}"
+                          )
+        else:
+            logger.success("All models that were supposed to be trained were trained. Awesome!")
+
 
     def clean_up_matrix_building_tables(self):
-        logging.info("Cleaning up cohort and labels tables")
+        logger.debug("Cleaning up cohort and labels tables")
         with timeout(self.cleanup_timeout):
             self.cohort_table_generator.clean_up()
             self.label_generator.clean_up(self.labels_table_name)
+        logger.debug("Cleaning up cohort and labels tables: completed")
 
     def clean_up_subset_tables(self):
-        logging.info("Cleaning up cohort and labels tables")
+        logger.debug("Cleaning up cohort and labels tables")
         with timeout(self.cleanup_timeout):
             for subset_task in self.subset_tasks:
                 subset_task["subset_table_generator"].clean_up()
+        logger.debug("Cleaning up cohort and labels tables: completed")
 
     def _run_profile(self):
         cp = cProfile.Profile()
@@ -856,9 +896,8 @@ class ExperimentBase(ABC):
         with store.open('wb') as fd:
             cp.create_stats()
             marshal.dump(cp.stats, fd)
-            logging.info("Profiling stats of this Triage run calculated and written to %s"
-                         "in cProfile format.",
-                         store)
+            logger.spam(f"Profiling stats of this Triage run calculated and written to {store}"
+                         f"in cProfile format.")
 
     @experiment_entrypoint
     def run(self):
@@ -868,7 +907,7 @@ class ExperimentBase(ABC):
             else:
                 self._run()
         except Exception:
-            logging.exception("Run interrupted by uncaught exception")
+            logger.exception("Run interrupted by uncaught exception")
             raise
 
     __call__ = run
